@@ -89,6 +89,10 @@ project_se <- function(source, groupby, assignments,
   UseMethod("project_se", source)
 }
 
+#' @export
+#' @rdname project_se
+aggregate_se <- project_se
+
 #' @export
 project_se.relop <- function(source, groupby, assignments,
                              env = parent.frame()) {
@@ -154,6 +158,10 @@ project_nse <- function(source, groupby, ...,
   UseMethod("project_nse", source)
 }
 
+#' @export
+#' @rdname project_nse
+aggregate_nse <- project_nse
+
 #' @export
 project_nse.relop <- function(source, groupby, ...,
                               env = parent.frame()) {
